@@ -12,7 +12,7 @@
  *  You may distribute this under the terms of either the GNU General Public
  *  License or the Artistic License, as specified in the Perl README file.
  *
- *  $Id: dbdimp.h 7931 2006-10-10 19:58:53Z capttofu $
+ *  $Id: dbdimp.h 8436 2006-12-23 21:36:44Z capttofu $
  */
 
 /*
@@ -28,12 +28,14 @@
  * the server will be used to process prepare
  * statements as opposed to emulation in the driver
 */
+#define SQL_STATE_VERSION 40101
 #define MULTIPLE_RESULT_SET_VERSION 40102
 #define SERVER_PREPARE_VERSION 40103
 #define LIMIT_PLACEHOLDER_VERSION 50100
 #define GEO_DATATYPE_VERSION 50007
 #define NEW_DATATYPE_VERSION 50003
 #define SSL_VERIFY_VERSION 50023
+#define MYSQL_VERSION_5_0 50001
 
 /*
  *  The following are return codes passed in $h->err in case of
@@ -274,7 +276,11 @@ struct imp_sth_st {
 #endif
 
 #include <dbd_xsh.h>
-void	 do_error (SV* h, int rc, const char *what);
+#if MYSQL_VERSION_ID >= SQL_STATE_VERSION
+void    do_error (SV* h, int rc, const char *what, const char *sqlstate);
+#else
+void    do_error (SV* h, int rc, const char *what);
+#endif
 SV	*dbd_db_fieldlist (MYSQL_RES* res);
 
 void    dbd_preparse (imp_sth_t *imp_sth, SV *statement);
@@ -314,3 +320,4 @@ extern MYSQL* mysql_dr_connect(SV*, MYSQL*, char*, char*, char*, char*, char*,
 			       char*, imp_dbh_t*);
 
 extern int mysql_db_reconnect(SV*);
+int mysql_st_free_result_sets (SV * sth, imp_sth_t * imp_sth);
