@@ -9,7 +9,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '3.0008';
+$VERSION = '4.00';
 
 bootstrap DBD::mysql $VERSION;
 
@@ -25,12 +25,12 @@ sub driver{
     $class .= "::dr";
 
     # not a 'my' since we use it above to prevent multiple drivers
-    $drh = DBI::_new_drh($class, 
-        { 'Name' => 'mysql',
-        'Version' => $VERSION,
-        'Err'    => \$DBD::mysql::err,
-        'Errstr' => \$DBD::mysql::errstr,
-        'Attribution' => 'DBD::mysql by Rudy Lippan and Patrick Galbraith' });
+    $drh = DBI::_new_drh($class, { 'Name' => 'mysql',
+				   'Version' => $VERSION,
+				   'Err'    => \$DBD::mysql::err,
+				   'Errstr' => \$DBD::mysql::errstr,
+				   'Attribution' => 'DBD::mysql by Patrick Galbraith'
+				 });
 
     $drh;
 }
@@ -397,11 +397,13 @@ sub column_info {
 	}
 	elsif ($basetype =~ /date|time/) { # date/datetime/time/timestamp
 	    if ($basetype eq 'time' or $basetype eq 'date') {
-		$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
+		#$info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TYPE_TIME() : SQL_TYPE_DATE();
+                $info->{DATA_TYPE}   = ($basetype eq 'time') ? SQL_TIME() : SQL_DATE(); 
 		$info->{COLUMN_SIZE} = ($basetype eq 'time') ? 8 : 10;
 	    }
 	    else { # datetime/timestamp
-		$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
+		#$info->{DATA_TYPE}     = SQL_TYPE_TIMESTAMP();
+		$info->{DATA_TYPE}     = SQL_TIMESTAMP();
 		$info->{SQL_DATA_TYPE} = SQL_DATETIME();
 	        $info->{SQL_DATETIME_SUB} = $info->{DATA_TYPE} - ($info->{SQL_DATA_TYPE} * 10);
 		$info->{COLUMN_SIZE}   = ($basetype eq 'datetime') ? 19 : $type_params[0] || 14;
@@ -473,7 +475,7 @@ DBD::mysql - MySQL driver for the Perl5 Database Interface (DBI)
     @databases = DBI->data_sources("mysql");
        or
     @databases = DBI->data_sources("mysql",
-				   {"host" => $host, "port" => $port});
+      {"host" => $host, "port" => $port, "user" => $user, password => $pass});
 
     $sth = $dbh->prepare("SELECT * FROM foo WHERE bla");
        or
@@ -621,15 +623,18 @@ A C<database> must always be specified.
 
 =item port
 
-The hostname, if not specified or specified as '', will default to an
-MySQL daemon running on the local machine on the default port
-for the UNIX socket.
+The hostname, if not specified or specified as '' or 'localhost', will
+default to a MySQL server running on the local machine using the default for
+the UNIX socket. To connect to a MySQL server on the local machine via TCP,
+you must specify the loopback IP address (127.0.0.1) as the host.
 
-Should the MySQL daemon be running on a non-standard port number,
+Should the MySQL server be running on a non-standard port number,
 you may explicitly state the port number to connect to in the C<hostname>
 argument, by concatenating the I<hostname> and I<port number> together
 separated by a colon ( C<:> ) character or by using the  C<port> argument.
 
+To connect to a MySQL server on localhost using TCP/IP, you must specify the
+hostname as 127.0.0.1 (with the optional port).
 
 =item mysql_client_found_rows
 
@@ -740,23 +745,33 @@ disallow LOCAL.)
 
 =item Prepared statement support (server side prepare)
 
+As of 3.0002_1, server side prepare statements were on by default (if your
+server was >= 4.1.3). As of 3.0009, they were off by default again due to 
+issues with the prepared statement API (all other mysql connectors are
+set this way until C API issues are resolved). The requirement to use
+prepared statements still remains that you have a server >= 4.1.3
+
 To use server side prepared statements, all you need to do is set the variable 
 mysql_server_prepare in the connect:
 
 $dbh = DBI->connect(
-                    "DBI:mysql:database=test;host=localhost:mysql_server_prepare=1",
+                    "DBI:mysql:database=test;host=localhost;mysql_server_prepare=1",
                     "",
                     "",
                     { RaiseError => 1, AutoCommit => 1 }
                     );
 
+* Note: delimiter for this param is ';'
+
+There are many benefits to using server side prepare statements, mostly if you are 
+performing many inserts because of that fact that a single statement is prepared 
+to accept multiple insert values.
+
 To make sure that the 'make test' step tests whether server prepare works, you just
 need to export the env variable MYSQL_SERVER_PREPARE:
 
 export MYSQL_SERVER_PREPARE=1
 
-Test first without server side prepare, then with.
-
 
 =item mysql_embedded_options
 
@@ -795,17 +810,12 @@ $testdsn="DBI:mysqlEmb:database=test;mysql_embedded_groups=embedded_server,commo
     @dbs = $drh->func($hostname, $port, '_ListDBs');
     @dbs = $dbh->func('_ListDBs');
 
-Returns a list of all databases managed by the MySQL daemon
-running on C<$hostname>, port C<$port>. This method
-is rarely needed for databases running on C<localhost>: You should
-use the portable method
+Returns a list of all databases managed by the MySQL server
+running on C<$hostname>, port C<$port>. This is a legacy
+method.  Instead, you should use the portable method
 
     @dbs = DBI->data_sources("mysql");
 
-whenever possible. It is a design problem of this method, that there's
-no way of supplying a host name or port number to C<data_sources>, that's
-the only reason why we still support C<ListDBs>. :-(
-
 =back
 
 
@@ -879,7 +889,7 @@ The DBD::mysql driver supports the following attributes of database
 handles (read only):
 
   $errno = $dbh->{'mysql_errno'};
-  $error = $dbh->{'mysql_error};
+  $error = $dbh->{'mysql_error'};
   $info = $dbh->{'mysql_hostinfo'};
   $info = $dbh->{'mysql_info'};
   $insertid = $dbh->{'mysql_insertid'};
@@ -952,6 +962,23 @@ for $dbh using several ways:
 It is possible to set/unset the C<mysql_use_result> attribute after 
 creation of statement handle. See below.
 
+=item mysql_enable_utf8
+
+This attribute determines whether DBD::mysql should assume strings
+stored in the database are utf8.  This feature defaults to off.
+
+When set, a data retrieved from a textual column type (char, varchar,
+etc) will have the UTF-8 flag turned on if necessary.  This enables
+character semantics on that string.  You will also need to ensure that
+your database / table / column is configured to use UTF8.  See Chapter
+10 of the mysql manual for details.
+
+Additionally, turning on this flag tells MySQL that incoming data should
+be treated as UTF-8.  This will only take effect if used as part of the
+call to connect().  If you turn the flag on after connecting, you will
+need to issue the command C<SET NAMES utf8> to get the same effect.
+
+This option is experimental and may change in future versions.
 
 =head1 STATEMENT HANDLES
 
@@ -1190,6 +1217,68 @@ indication of such loss.
 
 =back
 
+=over
+
+=head1 MULTIPLE RESULT SETS
+
+As of version 3.0002_5, DBD::mysql supports multiple result sets (Thanks
+to Guy Harrison!). This is the first release of this functionality, so 
+there may be issues. Please report bugs if you run into them!
+
+The basic usage of multiple result sets is
+
+  do 
+  {
+    while (@row= $sth->fetchrow_array())
+    {
+      do stuff;
+    }
+  } while ($sth->more_results)
+
+An example would be:
+
+  $dbh->do("drop procedure if exists someproc") or print $DBI::errstr;
+
+  $dbh->do("create procedure somproc() deterministic
+   begin
+   declare a,b,c,d int;
+   set a=1;
+   set b=2;
+   set c=3;
+   set d=4;
+   select a, b, c, d;
+   select d, c, b, a;
+   select b, a, c, d;
+   select c, b, d, a;
+  end") or print $DBI::errstr;
+
+  $sth=$dbh->prepare('call someproc()') || 
+  die $DBI::err.": ".$DBI::errstr;
+
+  $sth->execute || die DBI::err.": ".$DBI::errstr; $rowset=0;
+  do {
+    print "\nRowset ".++$i."\n---------------------------------------\n\n";
+    foreach $colno (0..$sth->{NUM_OF_FIELDS}) {
+      print $sth->{NAME}->[$colno]."\t";
+    }
+    print "\n";
+    while (@row= $sth->fetchrow_array())  {
+      foreach $field (0..$#row) {
+        print $row[$field]."\t";
+      }
+      print "\n";
+    }
+  } until (!$sth->more_results)
+ 
+For more examples, please see the eg/ directory. This is where helpful
+DBD::mysql code snippits will be added in the future.
+
+=head2 Issues with Multiple result sets
+
+So far, the main issue is if your result sets are "jagged", meaning, the
+number of columns of your results vary. Varying numbers of columns could
+result in your script crashing. This is something that will be fixed soon.
+
 
 =head1 MULTITHREADING
 
@@ -1259,7 +1348,7 @@ Then enter the following commands (note - versions are just examples):
 
   cd ..
   gzip -cd Data-ShowTable-(version).tar.gz | tar xf -
-  cd Data-ShowTable-(version)
+  cd Data-ShowTable-3.3
   perl Makefile.PL
   make
   make install
@@ -1279,7 +1368,6 @@ then enter the header directory F</usr>, likewise for
 F</usr/lib/mysql/libmysqlclient.a> or F</usr/lib/libmysqlclient.so>.
 
 
-
 =head1 WIN32 INSTALLATION
 
 If you are using ActivePerl, you may use ppm to install DBD-mysql.
@@ -1306,7 +1394,7 @@ have a C compiler, the file README.win32 from the Perl source
 distribution tells you where to obtain freely distributable C compilers
 like egcs or gcc. The Perl sources are available via CPAN search
 
-    http://search.cpan.org
+  http://search.cpan.org
 
 I recommend using the win32clients package for installing DBD::mysql
 under Win32, available for download on www.tcx.se. The following steps
@@ -1418,15 +1506,11 @@ in the PPM program.
 
 =head1 AUTHORS
 
-A good part of the current version of B<DBD::mysql> is written
-by Jochen Wiedmann, then was maintained by
-Rudy Lippan (I<rlippan@remotelinux.com>), and Prepared Statement
-code written by Alexey Stroganov and Patrick Galbraith, and now 
-maintained by Patrick Galbraith (I<patg@mysql.com>), with the
-help of various people in the community. The first version's author
-was Alligator Descartes (I<descarte@symbolstone.org>), who has been
-aided and abetted by Gary Shea, Andreas Knig and Tim Bunce
-amongst others.
+The current version of B<DBD::mysql> is almost completely written
+by Jochen Wiedmann, and is now being maintained by
+Patrick Galbraith (I<patg@mysql.com>). 
+The first version's author was Alligator Descartes, who was aided
+and abetted by Gary Shea, Andreas Knig and Tim Bunce amongst others.
 
 The B<Mysql> module was originally written by Andreas Knig
 <koenig@kulturbox.de>. The current version, mainly an emulation
@@ -1451,37 +1535,33 @@ This module is maintained and supported on a mailing list,
 
     perl@lists.mysql.com
 
-To subscribe to this list, send a mail to
+To subscribe to this list, go to
 
-    perl-subscribe@lists.mysql.com
-
-or
-
-    perl-digest-subscribe@lists.mysql.com
+http://lists.mysql.com/perl?sub=1
 
 Mailing list archives are available at
 
-    http://www.progressive-comp.com/Lists/?l=msql-mysql-modules
-
+http://lists.mysql.com/perl
 
 Additionally you might try the dbi-user mailing list for questions about
 DBI and its modules in general. Subscribe via
 
-    http://www.fugue.com/dbi
+dbi-users-subscribe@perl.org
 
 Mailing list archives are at
 
-     http://www.rosat.mpe-garching.mpg.de/mailing-lists/PerlDB-Interest/
-     http://outside.organic.com/mail-archives/dbi-users/
-     http://www.coe.missouri.edu/~faq/lists/dbi.html
+http://groups.google.com/group/perl.dbi.users?hl=en&lr=
 
+Also, the main DBI site is at
+
+http://dbi.perl.org/
 
 =head1 ADDITIONAL DBI INFORMATION
 
 Additional information on the DBI project can be found on the World
 Wide Web at the following URL:
 
-    http://www.symbolstone.org/technology/perl/DBI
+    http://dbi.perl.org
 
 where documentation, pointers to the mailing lists and mailing list
 archives and pointers to the most current versions of the modules can
@@ -1493,6 +1573,16 @@ Information on the DBI interface itself can be gained by typing:
 
 right now!
 
+
+=head1 BUG REPORTING, ENHANCEMENT/FEATURE REQUESTS
+
+Please report bugs, including all the information needed
+such as DBD::mysql version, MySQL version, OS type/version, etc
+to this link:
+
+http://bugs.mysql.com/
+
+
 =cut
 
 
